//includes
#include <iostream>
#include <fstream>
#include <vector>

//functions
double prey(double alpha, double beta, double r, double f)
{
	return alpha * r - beta * r * f;
}

double predator(double delta, double gamma, double r, double f)
{
	return delta * r * f - gamma * f;
}

void euler(double & r0, double & f0, double start, double end, double deltat, std::vector<double> & parameters)
{
	double alpha = parameters[0];
	double beta  = parameters[1];
	double delta = parameters[2];
	double gamma = parameters[3];
	double deltar;	
	double deltaf;
	for (double t=start; t < end; t += deltat)
	{
		deltar = prey(alpha,beta, r0, f0) * deltat;
		deltaf = predator(delta,gamma,r0,f0) * deltat;
		r0 += deltar;
		f0 += deltaf;
	}
}	

//main
int main(int argc, char **argv)
{
	//start parameters
	std::string myfilename = "eulerdata.txt";
	double r0 = 500.0;
	double f0 = 20.0;
	double alpha = 0.04;
	double beta=0.005;
	double delta = beta*0.075;
	double gamma = 0.12;
	double deltat = 0.1;
	std::cout << "syntax: euler < myfilename rabbits foxes > " << std::endl;
	if(argc>3)
	{
		myfilename = argv[1];
		r0 = std::stod(argv[2]);
		f0 = std::stod(argv[3]);
	}

	std::vector<double> parameters {alpha,beta,delta,gamma};	
	std::ofstream output(myfilename);
	if (output.is_open())
	{
		for (double time=1.0; time < 1000.0; time += 1.0)
		{
			euler(r0,f0, time-1.0, time, deltat, parameters);
			output << time << " " << r0 << " " << f0 << std::endl; 
			//std::cout << time << " " << r0 << " " << f0 << std::endl; 
		}
		std::cout << "run finished: data saved in " << myfilename << std::endl;
		output.close();
	}
	return 0;
}
