// Rational.cpp
// hold implementation of all definitions of the Rational class
//
// (c) Peter Beerli
//
#include <iostream>
#include "Rational.h"


// Greatest common divisor
// using Euclidian algorithm from Wikipedia
// https://en.wikipedia.org/wiki/Euclidean_algorithm
// (extracted on Oct 18 2016)
int gcd(const int p, const int q)
{
	int pp = p;
	int qq = q;

        //  while b ≠ 0
	while(qq != 0)
	{
	        //    t := b;
		int t = qq;	 
	        //    b := a mod b;
		qq = pp % qq;		 
        	//    a := t; 
 		pp = t;
	}	
        return pp;
}

Rational::Rational(int newnom, int newdenom)
{
	num = newnom;
	denom = newdenom;
	reduce();
}

Rational::Rational(const Rational & orig)
{
	this->operator=(orig);
}

void Rational::reduce()
{
	int x = gcd(num,denom);
	num = num /x;
	denom = denom /x ;
}

const Rational & Rational::operator=(const Rational &rhs)
{
	if(this != &rhs)
	{
		num = rhs.num;
		denom = rhs.denom;
	}
	return *this;
}


const Rational Rational::operator*(const Rational &other)
{
	Rational tmp;
	tmp.num = num * other.num;
	tmp.denom = denom * other.denom;
	tmp.reduce();
	return tmp;
}



void Rational::print()
{
	std::cout << num <<'/'<<denom<<std::endl;
}

