// a simple square root calculator
// 
// (c) Peter Beerli, Tallahassee, FL 2016
//
//
// This program is interactive when started with no argument. 
// If a argument is given then it will only return the 
// squareroot of the argument (if it is a number)
// Without an argument it will ask for numbers on the terminal.
//
// Syntax: 
//  root 
//  root number
//
// the function and its derivative to solve
// x^2 = a numerically , first we set 0==x^2 - a 
// this allows to to calculate
// f(x,a) = x^2 - a
// f'(x,a) = 2x
// the newton method uses the tangent equation to find new approximation of x
// until f(x,a) reaches zero, the x is then the result:
// x_{n} = x_{n-1} - f(x_{n-1})/f'(x_{n-1})
//
#include <iostream>
#include <cmath>


// newton() takes an argument x
double newton(double x0, double a)
{
  // loop until we have a precision of 0.00001
  double x = x0;
  double precision = 0.0001;
  double fx = 10000.0;
  double dfx;
  while (fabs(fx) > precision)
    {
      fx = x*x -a;
      dfx = 2.0 * x;
      //std::cout << fx << " " << dfx << '\n';
      x = x - fx / dfx;
    }
  return x;
}


// start main and evaluate whether there is an argument or not
int main(int argc, char ** argv)
{
  double x0 = 1.0;
  if (argc > 1)
    {
      double a = std::stof(argv[1]);
      double y = newton(x0,a);
      std::cout << "Sqrt(" << a << ")=" << y << std::endl;
      return 0;
    }
  else
    {
      // not yet 
      std::string input;
      std::cout << "Enter a positive number to get the square root or -1 to quit\n";
      std::cin >> input;
      double a =std::stod(input);
      while (a > 0.0)
	{ 
	  double y = newton(x0,a);
	  std::cout << "Sqrt(" << x << ")=" << y << std::endl;
	  std::cout << "Enter a positive number to get the square root or -1 to quit\n";
	  std::cin >> input;
	  x =std::stod(input);
	}
      return 0;
    }
}
 
