/*
	class_root.cpp (c) Dennis E. Slice

	Program to implement and compare results of Newton's method and bisection method
	for root finding.
*/

#include <iostream>
#include <cmath>

using namespace std;

// The function.
double f(double x) 
{
	return 10*x*x - 3*x;
}

// The derivative of f(x)
double df(double x)
{
	return 20*x - 3;
}

// Newton method.
double newton(double x, double tol, int n) 
{
	double fx = f(x);
	double dfx = 0.0;
	int count = 0;

	// Iterate 
	cout << "Count\tx\tf(x)" << endl;
	while ((fabs(fx)>tol) && (count++ < n) ) 
	{
		fx = f(x);
		dfx = df(x);
		cout << count << "\t" << x << "\t" << fx << endl; 
		x = x - fx/dfx;
	}
	cout << "\t====" <<endl;
	return x;
}


// Bisection method.
double bisection(double a, double b, double tol, int n) 
{
	double c = (a+b)/2;
	double fa = f(a);
	double fb = f(b);
	double fc = f(c);
	int count = 0;

	cout << "Count\ta\tc\tb | fa\tfc\tfb" << endl;
	while ( ((b-a)>tol) && (count++ < n) ) 
	{
		c = (a+b)/2;
		fc = f(c);
		cout << count << "\t" << a<< "\t" << c<< "\t" << b << " | " << fa << "\t"<< fc << "\t" << fb << endl;
		if (fa < fb)
		{
			if (fc<0.0) 
			{
				a = c;
				fa = fc;
			} else
			{
				b = c;
				fb = fc;
			}
		} else 
		{
			if (fc>0.0) 
			{
				b = c;
				fb = fc;
			} else
			{
				a = c;
				fa = fc;
			}
		}
	}

	cout << "\t====" <<endl;
	return c;
}

// Main program method.
int main(int argc, char ** argv) 
{
	double root = -999.0;

	double tol = 0.00001;
	double a = 0.1;
	double b = 0.6;
	int n = 100;



	cout << endl;

	// Solve using Newton. 
	cout << "Newton's method..." << endl;
	root = newton(b, tol, n);
	cout << "\troot= " << root << endl;
	cout << endl;

	// Solve using bisection.
	cout << "Bisection method..." << endl;
	root = bisection(a, b, tol, n);
	cout << "\troot= " << root << endl;

	cout << endl;
	return 0;
}

